from cffi import FFI
import os

module_name = '_smbus_cffi'

ffi = FFI()

ffi.cdef("""
typedef unsigned char __u8;
typedef int32_t __s32;
typedef unsigned short int __u16;

#define I2C_SLAVE ...
#define I2C_PEC ...

/* smbus_access read or write markers */
#define I2C_SMBUS_READ  ...
#define I2C_SMBUS_WRITE ...

/* SMBus transaction types (size parameter in the above functions)
   Note: these no longer correspond to the (arbitrary) PIIX4 internal codes! */
#define I2C_SMBUS_QUICK             ...
#define I2C_SMBUS_BYTE              ...
#define I2C_SMBUS_BYTE_DATA         ...
#define I2C_SMBUS_WORD_DATA         ...
#define I2C_SMBUS_PROC_CALL         ...
#define I2C_SMBUS_BLOCK_DATA        ...
#define I2C_SMBUS_I2C_BLOCK_BROKEN  ...
#define I2C_SMBUS_BLOCK_PROC_CALL   ...   /* SMBus 2.0 */
#define I2C_SMBUS_I2C_BLOCK_DATA    ...

/*
 * Data for SMBus Messages
 */
#define I2C_SMBUS_BLOCK_MAX         ...    /* As specified in SMBus standard */
union i2c_smbus_data {
        __u8 byte;
        __u16 word;
        __u8 block[34]; /* block[0] is used for length */
                        /* and one more for PEC */
};

static inline __s32 i2c_smbus_access(int file, char read_write, __u8 command, int size, union i2c_smbus_data *data);

static inline __s32 i2c_smbus_write_quick(int file, __u8 value);

static inline __s32 i2c_smbus_read_byte(int file);
static inline __s32 i2c_smbus_write_byte(int file, __u8 value);

static inline __s32 i2c_smbus_read_byte_data(int file, __u8 command);
static inline __s32 i2c_smbus_write_byte_data(int file, __u8 command, __u8 value);

static inline __s32 i2c_smbus_read_word_data(int file, __u8 command);
static inline __s32 i2c_smbus_write_word_data(int file, __u8 command, __u16 value);

static inline __s32 i2c_smbus_process_call(int file, __u8 command, __u16 value);

//static inline __s32 i2c_smbus_read_block_data(int file, __u8 command, __u8 *values)
//static inline __s32 i2c_smbus_write_block_data(int file, __u8 command, __u8 length, const __u8 *values)
""")

include_dir = os.path.join(os.path.dirname(__file__), 'include')

ffi.set_source(module_name, """
#include <sys/types.h>
#include <linux/i2c-dev.h>
""", include_dirs=[include_dir])

if __name__ == '__main__':
    ffi.compile()



# smbus integration test.
# Test the smbus-cffi implementation using an arduino sketch that implements
# the smbus protocol using Wire.h. See test/test_sketch

from serial import Serial
from smbus import SMBus
from smbus.util import int2byte
from smbus.util import PY3K
import time
import py

if PY3K:
    def byte2int(x):
        return x
else:
    def byte2int(x):
        return ord(x)


PORT = '/dev/ttyACM0'
ADDR = 0x04
BUS = 1

# Commands
GETDATA = int2byte(254)
RESET = int2byte(255)

# Testcases
WRITE_QUICK = 1
READ_BYTE = 2
WRITE_BYTE = 3
READ_BYTE_DATA = 4
WRITE_BYTE_DATA = 5
READ_WORD_DATA = 6
WRITE_WORD_DATA = 7
PROCESS_CALL = 8
READ_BLOCK_DATA = 9
WRITE_BLOCK_DATA = 10
BLOCK_PROCESS_CALL = 11
READ_I2C_BLOCK_DATA = 12
WRITE_I2C_BLOCK_DATA = 13

DELAY = 1

i2c_features = {}
i2c_feature_map = {
    'SMBus Quick Command': WRITE_QUICK,
    'SMBus Send Byte': WRITE_BYTE,
    'SMBus Receive Byte': READ_BYTE,
    'SMBus Write Byte': WRITE_BYTE_DATA,
    'SMBus Read Byte': READ_BYTE_DATA,
    'SMBus Write Word': WRITE_WORD_DATA,
    'SMBus Read Word': READ_WORD_DATA,
    'SMBus Process Call': PROCESS_CALL,
    'SMBus Block Write': WRITE_BLOCK_DATA,
    'SMBus Block Read': READ_BLOCK_DATA,
    'SMBus Block Process Call': BLOCK_PROCESS_CALL,
    'SMBus PEC': -1,
    'I2C Block Write': WRITE_I2C_BLOCK_DATA,
    'I2C Block Read': READ_I2C_BLOCK_DATA,
}


def get_i2c_features():
    import subprocess
    try:
        output = subprocess.Popen(["i2cdetect", "-F", "1"],
                                  stdout=subprocess.PIPE).communicate()[0]
        return (i.decode('utf-8') for i in output.split(b'\n')[1:])
    except OSError:
        py.test.skip("Requires i2cdetect command")


def detect_i2c_features():
    features = get_i2c_features()
    for line in features:
        if line == '':
            continue
        for n in ('yes', 'no'):
            i = line.find(n)
            if i == -1:
                continue
            key = line[0:i - 1].strip()
            key = i2c_feature_map.get(key, -1)
            i2c_features[key] = True if n == 'yes' else False
            break
        else:
            import pdb
            pdb.set_trace()

detect_i2c_features()
del detect_i2c_features


def command(testcase):
    def wrapper(f):
        f.testcase = testcase
        return f
    return wrapper


def pytest_funcarg__smbus(*args, **kwargs):
    return SMBus(BUS)


def test_open():
    bus = SMBus()
    py.test.raises(IOError, 'bus.open(-13)')

    bus.open(BUS)  # does not raise

    if hasattr(bus, '_fd'):
        assert bus._fd != -1


def test_write_quick(smbus):
    py.test.raises(TypeError, "smbus.write_quick('a')")
    py.test.raises(IOError, "smbus.write_quick(44)")
    smbus.write_quick(ADDR)


def test_pec(smbus):
    assert not smbus.pec  # default value
    smbus.pec = None
    assert not smbus.pec
    smbus.pec = 5
    assert smbus.pec
    smbus.pec = True
    assert smbus.pec


class BaseSMBusIntegration(object):
    def setup_method(self, meth):
        if not i2c_features[meth.testcase]:
            py.test.skip("smbus feature not supported")
        self.bus = SMBus(BUS)
        self.serial = Serial(PORT, 9600)
        self.serial.write(RESET)
        time.sleep(DELAY)
        self.serial.write(int2byte(meth.testcase))
        time.sleep(DELAY)

    def getdata(self):
        self.serial.write(GETDATA)
        time.sleep(DELAY)
        return self.serial.readline().strip()


class TestSMBusIntegration(BaseSMBusIntegration):

    def setup_method(self, meth):
        BaseSMBusIntegration.setup_method(self, meth)
        if hasattr(self.bus, '_compat'):
            self.bus._compat = True

    @command(WRITE_QUICK)
    def test_write_quick(self):
        self.bus.write_quick(ADDR)
        data = self.getdata()
        testcase, numbytes = [int(i) for i in data.split(b"#")]
        assert WRITE_QUICK == testcase
        assert 0 == numbytes

    @command(READ_BYTE)
    def test_read_byte(self):
        self.serial.write(b'n')
        time.sleep(DELAY)
        byte = self.bus.read_byte(ADDR)
        'read ', byte
        data = int(self.getdata())
        assert data == READ_BYTE
        assert byte == ord('n')

    @command(READ_BYTE)
    def test_read_byte2(self):
        self.serial.write(int2byte(20))
        time.sleep(DELAY)
        byte = self.bus.read_byte(ADDR)
        'read ', byte
        data = int(self.getdata())
        assert data == READ_BYTE
        assert byte == 20

    @command(WRITE_BYTE)
    def test_write_byte(self):
        byte = 31
        self.bus.write_byte(ADDR, byte)
        data = self.getdata()
        testcase, numbytes, bytes = [int(i) for i in data.split(b"#")]
        assert testcase == WRITE_BYTE
        assert numbytes == 1
        assert bytes == 31

    @command(READ_BYTE_DATA)
    def test_read_byte_data(self):
        cmd = 17
        byte = self.bus.read_byte_data(ADDR, cmd)
        data = self.getdata()
        testcase, command, byte2 = [int(i) for i in data.split(b"#")]
        assert testcase == READ_BYTE_DATA
        assert 17 == command
        assert byte == byte2

    @command(WRITE_BYTE_DATA)
    def test_write_byte_data(self):
        cmd = 17
        byte = 235
        self.bus.write_byte_data(ADDR, cmd, byte)
        time.sleep(DELAY)
        data = self.getdata()
        testcase, numbytes, command, byte2 = [int(i) for i in data.split(b"#")]
        assert testcase == WRITE_BYTE_DATA
        assert 2 == numbytes
        assert cmd == command
        assert byte == byte2

    @command(READ_WORD_DATA)
    def test_read_word_data(self):
        bytes = [1 << 6, 1 << 7]
        word = bytes[1] << 8 | bytes[0]
        cmd = 27
        self.serial.write(int2byte(bytes[0]) + int2byte(bytes[1]))
        time.sleep(DELAY)
        worddata = self.bus.read_word_data(ADDR, cmd)
        assert word == worddata
        data = self.getdata()
        testcase, cmd2, worddata2 = data.split(b"#")
        assert int(testcase) == READ_WORD_DATA
        assert cmd == int(cmd2)
        worddata2 = [byte2int(i) for i in worddata2]
        assert word == worddata2[1] << 8 | worddata2[0]

    @command(WRITE_WORD_DATA)
    def test_write_word_data(self):
        word = 0x40C8
        cmd = 13
        self.bus.write_word_data(ADDR, cmd, word)
        data = self.getdata()
        testcase, numbytes, reg, worddata = data.split(b"#")
        assert int(testcase) == WRITE_WORD_DATA
        assert int(reg) == cmd
        assert int(numbytes) == 3
        worddata = [int(i) for i in worddata.split(b'|')]
        assert word == worddata[1] << 8 | worddata[0]

    @command(PROCESS_CALL)
    def test_process_call(self):
        word = 0xFADE
        cmd = 123
        result = self.bus.process_call(ADDR, cmd, word)
        data = self.getdata()
        testcase, cmd2, byte1, byte2 = [int(i) for i in data.split(b"#")]
        if hasattr(self.bus, '_compat') and not self.bus._compat: # in compat mode we do not get a return value
            assert result == 0xCAFE
        assert testcase == PROCESS_CALL
        assert cmd == cmd2
        assert word == byte1 | byte2 << 8

    @command(READ_BLOCK_DATA)
    def test_read_block_data(self):
        cmd = 217
        self.bus.read_block_data(ADDR, cmd)
        data = self.getdata()
        testcase, register = [int(i) for i in data.split(b"#")]
        assert testcase == READ_BLOCK_DATA
        assert cmd == register
        assert 0, 'incomplete'

    @command(BLOCK_PROCESS_CALL)
    def test_block_process_call(self):
        assert 0, 'incomplete'

    @command(WRITE_BLOCK_DATA)
    def test_write_block_data(self):
        data = list(range(65, 86))
        cmd = 0x7
        self.bus.write_block_data(ADDR, cmd, data)
        d = self.getdata()
        testcase, numbytes, reg, blockdata = d.split(b"#")
        assert WRITE_BLOCK_DATA == int(testcase)
        assert cmd == int(reg)
        blockdata = [int(i) for i in blockdata.split(b'|')]
        assert blockdata == [len(data)] + data

    @command(READ_I2C_BLOCK_DATA)
    def test_read_i2c_block_data(self):
        cmd = 217
        exp = list(range(100, 132))
        blockdata = self.bus.read_i2c_block_data(ADDR, cmd)
        data = self.getdata()
        testcase, reg, numbytes = [int(i) for i in data.split(b"#")]
        assert READ_I2C_BLOCK_DATA == testcase
        assert cmd == reg
        assert 1 == numbytes
        assert exp == blockdata

    @command(WRITE_I2C_BLOCK_DATA)
    def test_write_i2c_block_data(self):
        cmd = 217
        exp = list(range(100, 131))
        self.bus.write_i2c_block_data(ADDR, cmd, exp)
        data = self.getdata()
        testcase, numbytes, reg, blockdata = data.split(b"#")
        blockdata = [int(i) for i in blockdata.split(b'|')]
        assert WRITE_I2C_BLOCK_DATA == int(testcase)
        assert cmd == int(reg)
        assert len(exp) + 1 == int(numbytes)
        assert exp == blockdata


class TestCompatMode(BaseSMBusIntegration):
    @command(PROCESS_CALL)
    def test_process_call(self):
        word = 0xFADE
        cmd = 123
        result = self.bus.process_call(ADDR, cmd, word)
        data = self.getdata()
        testcase, reg, byte1, byte2 = [int(i) for i in data.split(b"#")]
        # in compat mode we do not get a return value
        assert result is None
        assert testcase == PROCESS_CALL
        assert cmd == reg
        assert word == byte1 | byte2 << 8



__all__ = [
    "__title__", "__summary__", "__uri__", "__version__", "__author__",
    "__email__", "__license__",
]

__title__ = "mysmbus"
__summary__ = 'This Python module allows SMBus access through the I2C /dev interface on Linux hosts. The host kernel must have I2C support, I2C device interface support, and a bus adapter driver.'
__uri__ = 'https://github.com/bivab/smbus-cffi'

__version__ = "0.1"

__author__ = "David Schneider"
__email__ = "david.schneider@bivab.de"

__license__ = 'GPLv2'



from smbus import SMBus


class SMBusProxy(object):
    def __init__(self, bus=-1):
        self.called = False
        self.passed_args = ()
        self.smbus = SMBus()
        if bus != -1:
            self.open(bus)

    def open(self, *args):
        self.called = True
        self.passed_args = args

    def __getattr__(self, name, *args, **kwargs):
        return getattr(self.smbus, name, *args, **kwargs)


class TestSMBus_init(object):

    def test_init_with_arg_tries_to_open(self):
        bus = SMBusProxy(12)

        assert bus.called
        assert bus.passed_args == (12,)

    def test_init_does_nothing_by_default(self):
        bus = SMBusProxy()
        if hasattr(bus, '_fd'):
            assert bus._fd == -1
            assert bus._addr == -1
            assert bus._pec == 0
        assert not bus.called


def test_list_to_smbus_data():
    from smbus import ffi, list_to_smbus_data
    lst = list(range(10))
    data = ffi.new("union i2c_smbus_data *")
    list_to_smbus_data(data, lst)
    assert data.block[0] == 10
    for i in lst:
        assert data.block[i + 1] == i


def test_smbus_data_to_list():
    from smbus import ffi, list_to_smbus_data, smbus_data_to_list
    lst = list(range(10))
    data = ffi.new("union i2c_smbus_data *")
    list_to_smbus_data(data, lst)
    assert smbus_data_to_list(data) == list(range(10))



from smbus.util import validate
import py


@validate(a=int, b=str, c=float, d=dict, e=list)
def fn(a, b, c, d, e):
    return (a, b, c, d, e)


@validate(b=int, d=dict)
def fn2(a, b, c, d):
    return (a, b, c, d)


@validate(a=int, b=int, c=float)
def fn3(a, b, c=3.2):
    return (a, b, c)


def test_all():
    assert fn(1, '123', 1.0, {'a': 1}, [2]) == (1, '123', 1.0, {'a': 1}, [2])


def test_missing_args():
    py.test.raises(TypeError, "fn(1, '123', 1.0)")


def test_int():
    py.test.raises(TypeError, "fn(1.0, '123', 1.0, {'a': 2}, [1])")


def test_float():
    py.test.raises(TypeError, "fn(1, '123', '1', {'a': 2}, [1])")


def test_string():
    py.test.raises(TypeError, "fn(1, 123, '1', {'a': 2}, [1])")


def test_list():
    py.test.raises(TypeError, "fn(1, 123, '1', {'a': 2}, '1')")


def test_dict():
    py.test.raises(TypeError, "fn(1, 123, '1', 2, [1])")


def test_partial_spec():
    assert fn2(1.0, 1, 'asdf', {'a': 2}) == (1.0, 1, 'asdf', {'a': 2})


def test_default_arg():
    assert fn3(1, 2) == (1, 2, 3.2)
    assert fn3(1, 2, 4.1) == (1, 2, 4.1)
    py.test.raises(TypeError, "fn3(1, 123, '1')")


@py.test.mark.skipif("sys.version_info[0] < 3")
def test_default_kwarg():
    args = [1, 2]
    assert fn3(*args) == (1, 2, 3.2)
    # currently not supported assert fn3(*args, c=4) == (1, 2, 4)



import sys

PY3K = sys.version_info[0] == 3


if PY3K:
    # Based on six
    def int2byte(x):
        return bytes((x,))
else:
    int2byte = chr


def get_validator(tp, name):
    def f(x):
        if not isinstance(x, tp):
            raise TypeError("Expected %s" % name)
    return f


validators = {}
for tp, name in [(int, 'integer'), (float, 'float'),
                 (str, 'string'), (dict, 'dict'), (list, 'list')]:
    validators[tp] = get_validator(tp, name)


def validate(**schema):
    def wrapper(fn):
        code = fn.__code__
        nargs = code.co_argcount
        varnames = code.co_varnames
        if PY3K:
            defaults = fn.__defaults__ if fn.__defaults__ else []
            kwdefaults = fn.__kwdefaults__ if fn.__kwdefaults__ else {}
        else:
            defaults = fn.func_defaults if fn.func_defaults else []
            kwdefaults = {}

        def validator(*args):
            largs = len(args)

            if largs != nargs and largs + len(defaults) + len(kwdefaults) != nargs:
                raise TypeError("%s() takes exactly %d arguments (%d given)" %
                                (fn.__name__, nargs, len(args) + len(kwdefaults)))
            for i, value in enumerate(args):
                name = varnames[i]
                if name not in schema:
                    continue
                typ = schema[name]
                validators[typ](value)
            if largs < nargs:
                for i in range(largs, nargs):
                    value = defaults[largs - i]
                    name = varnames[i]
                    if name not in schema:
                        continue
                    typ = schema[name]
                    validators[typ](value)
            return fn(*args, **kwdefaults)
        validator.__name__ = fn.__name__
        validator.__doc__ = fn.__doc__
        return validator
    return wrapper



#! /usr/bin/env python 2.7 (62211)
#coding=utf-8
# Compiled at: 2017-05-20 07:14:44
#Powered by BugScaner
#http://tools.bugscaner.com/
#如果觉得不错,请分享给你朋友使用吧!
"""This module defines an object type that allows SMBus transactions
on hosts running the Linux kernel.  The host kernel must have I2C
support, I2C device interface support, and a bus adapter driver.
All of these can be either built-in to the kernel, or loaded from
modules.
 
Because the I2C device interface is opened R/W, users of this
module usually must have root permissions."""
import os
from .util import validate
from .util import int2byte
from fcntl import ioctl
from ._smbus_cffi import ffi
from ._smbus_cffi import lib as SMBUS
MAXPATH = 16
 
class SMBus(object):
    """SMBus([bus]) -> SMBus
    Return a new SMBus object that is (optionally) connected to the
    specified I2C device interface.
    """
    _fd = -1
    _addr = -1
    _pec = 0
    _compat = False
 
    def __init__(self, bus=-1):
        if bus >= 0:
            self.open(bus)
 
    def close(self):
        """close()
         
        Disconnects the object from the bus.
        """
        os.close(self._fd)
        self._fd = -1
        self._addr = -1
        self._pec = 0
 
    def dealloc(self):
        self.close()
 
    def open(self, bus):
        """open(bus)
         
        Connects the object to the specified SMBus.
        """
        bus = int(bus)
        path = '/dev/i2c-%d' % (bus,)
        if len(path) >= MAXPATH:
            raise OverflowError('Bus number is invalid.')
        try:
            self._fd = os.open(path, os.O_RDWR, 0)
        except OSError as e:
            raise IOError(e.errno)
 
    def _set_addr(self, addr):
        """private helper method"""
        if self._addr != addr:
            ioctl(self._fd, SMBUS.I2C_SLAVE, addr)
            self._addr = addr
 
    @validate(addr=int)
    def write_quick(self, addr):
        """write_quick(addr)
         
        Perform SMBus Quick transaction.
        """
        self._set_addr(addr)
        if SMBUS.i2c_smbus_write_quick(self._fd, SMBUS.I2C_SMBUS_WRITE) != 0:
            raise IOError(ffi.errno)
 
    @validate(addr=int)
    def read_byte(self, addr):
        """read_byte(addr) -> result
         
        Perform SMBus Read Byte transaction.
        """
        self._set_addr(addr)
        result = SMBUS.i2c_smbus_read_byte(self._fd)
        if result == -1:
            raise IOError(ffi.errno)
        return result
 
    @validate(addr=int, val=int)
    def write_byte(self, addr, val):
        """write_byte(addr, val)
         
        Perform SMBus Write Byte transaction.
        """
        self._set_addr(addr)
        if SMBUS.i2c_smbus_write_byte(self._fd, ffi.cast('__u8', val)) == -1:
            raise IOError(ffi.errno)
 
    @validate(addr=int, cmd=int)
    def read_byte_data(self, addr, cmd):
        """read_byte_data(addr, cmd) -> result
         
        Perform SMBus Read Byte Data transaction.
        """
        self._set_addr(addr)
        res = SMBUS.i2c_smbus_read_byte_data(self._fd, ffi.cast('__u8', cmd))
        if res == -1:
            raise IOError(ffi.errno)
        return res
 
    @validate(addr=int, cmd=int, val=int)
    def write_byte_data(self, addr, cmd, val):
        """write_byte_data(addr, cmd, val)
         
        Perform SMBus Write Byte Data transaction.
        """
        self._set_addr(addr)
        if SMBUS.i2c_smbus_write_byte_data(self._fd, ffi.cast('__u8', cmd), ffi.cast('__u8', val)) == -1:
            raise IOError(ffi.errno)
 
    @validate(servoid=int)
    def XiaoRGEEK_Read_ServoAngle(self, servo_id):
        """XiaoRGEEK_Read_ServoAngle(servo_id) -> result
        """
        self._set_addr(80)
        res = SMBUS.i2c_smbus_read_byte_data(self._fd, ffi.cast('__u8', servo_id))
        if res == -1:
            raise IOError(ffi.errno)
        return res
 
    @validate(servoid=int, angle=int)
    def XiaoRGEEK_SetServo(self, servo_id, angle):
        """XiaoRGEEK_SetServo(servo_id, angle)"""
        self._set_addr(80)
        if SMBUS.i2c_smbus_write_byte_data(self._fd, ffi.cast('__u8', servo_id), ffi.cast('__u8', angle)) == -1:
            raise IOError(ffi.errno)
 
    def XiaoRGEEK_ReSetServo(self):
        """XiaoRGEEK_ReSetServo()"""
        self._set_addr(80)
        if SMBUS.i2c_smbus_write_byte_data(self._fd, ffi.cast('__u8', 0), ffi.cast('__u8', 1)) == -1:
            raise IOError(ffi.errno)
 
    def XiaoRGEEK_SaveServo(self):
        """XiaoRGEEK_SaveServo()"""
        self._set_addr(80)
        if SMBUS.i2c_smbus_write_byte_data(self._fd, ffi.cast('__u8', 17), ffi.cast('__u8', 1)) == -1:
            raise IOError(ffi.errno)
 
    def XiaoRGEEK_ReadVol(self):
        """XiaoRGEEK_ReadVol() -> result
        """
        self._set_addr(80)
        res = SMBUS.i2c_smbus_read_byte_data(self._fd, ffi.cast('__u8', 18))
        if res == -1:
            raise IOError(ffi.errno)
        return res
 
    def XiaoRGEEK_SpeedCounter1(self):
        """XiaoRGEEK_SpeedCounter1() -> result
        """
        self._set_addr(80)
        res = SMBUS.i2c_smbus_read_byte_data(self._fd, ffi.cast('__u8', 19))
        if res == -1:
            raise IOError(ffi.errno)
        return res
 
    def XiaoRGEEK_SpeedCounter2(self):
        """XiaoRGEEK_SpeedCounter2() -> result
        """
        self._set_addr(80)
        res = SMBUS.i2c_smbus_read_byte_data(self._fd, ffi.cast('__u8', 20))
        if res == -1:
            raise IOError(ffi.errno)
        return res
 
    @validate(addr=int, cmd=int)
    def read_word_data(self, addr, cmd):
        """read_word_data(addr, cmd) -> result
         
        Perform SMBus Read Word Data transaction.
        """
        self._set_addr(addr)
        result = SMBUS.i2c_smbus_read_word_data(self._fd, ffi.cast('__u8', cmd))
        if result == -1:
            raise IOError(ffi.errno)
        return result
 
    @validate(addr=int, cmd=int, val=int)
    def write_word_data(self, addr, cmd, val):
        """write_word_data(addr, cmd, val)
         
        Perform SMBus Write Word Data transaction.
        """
        self._set_addr(addr)
        if SMBUS.i2c_smbus_write_word_data(self._fd, ffi.cast('__u8', cmd), ffi.cast('__u16', val)) == -1:
            raise IOError(ffi.errno)
 
    @validate(addr=int, cmd=int, val=int)
    def process_call(self, addr, cmd, val):
        """process_call(addr, cmd, val)
         
        Perform SMBus Process Call transaction.
         
        Note: although i2c_smbus_process_call returns a value, according to
        smbusmodule.c this method does not return a value by default.
         
        Set _compat = False on the SMBus instance to get a return value.
        """
        self._set_addr(addr)
        ret = SMBUS.i2c_smbus_process_call(self._fd, ffi.cast('__u8', cmd), ffi.cast('__u16', val))
        if ret == -1:
            raise IOError(ffi.errno)
        if self._compat:
            return ret
 
    @validate(addr=int, cmd=int)
    def read_block_data(self, addr, cmd):
        """read_block_data(addr, cmd) -> results
         
        Perform SMBus Read Block Data transaction.
        """
        self._set_addr(addr)
        data = ffi.new('union i2c_smbus_data *')
        if SMBUS.i2c_smbus_access(self._fd, int2byte(SMBUS.I2C_SMBUS_READ), ffi.cast('__u8', cmd), SMBUS.I2C_SMBUS_BLOCK_DATA, data):
            raise IOError(ffi.errno)
        return smbus_data_to_list(data)
 
    @validate(addr=int, cmd=int, vals=list)
    def write_block_data(self, addr, cmd, vals):
        """write_block_data(addr, cmd, vals)
         
        Perform SMBus Write Block Data transaction.
        """
        self._set_addr(addr)
        data = ffi.new('union i2c_smbus_data *')
        list_to_smbus_data(data, vals)
        if SMBUS.i2c_smbus_access(self._fd, int2byte(SMBUS.I2C_SMBUS_WRITE), ffi.cast('__u8', cmd), SMBUS.I2C_SMBUS_BLOCK_DATA, data):
            raise IOError(ffi.errno)
 
    @validate(addr=int, cmd=int, vals=list)
    def block_process_call(self, addr, cmd, vals):
        """block_process_call(addr, cmd, vals) -> results
         
        Perform SMBus Block Process Call transaction.
        """
        self._set_addr(addr)
        data = ffi.new('union i2c_smbus_data *')
        list_to_smbus_data(data, vals)
        if SMBUS.i2c_smbus_access(self._fd, SMBUS.I2C_SMBUS_WRITE, ffi.cast('__u8', cmd), SMBUS.I2C_SMBUS_BLOCK_PROC_CALL, data):
            raise IOError(ffi.errno)
        return smbus_data_to_list(data)
 
    @validate(addr=int, cmd=int, len=int)
    def read_i2c_block_data(self, addr, cmd, len=32):
        """read_i2c_block_data(addr, cmd, len=32) -> results
         
        Perform I2C Block Read transaction.
        """
        self._set_addr(addr)
        data = ffi.new('union i2c_smbus_data *')
        data.block[0] = len
        if len == 32:
            arg = SMBUS.I2C_SMBUS_I2C_BLOCK_BROKEN
        else:
            arg = SMBUS.I2C_SMBUS_I2C_BLOCK_DATA
        if SMBUS.i2c_smbus_access(self._fd, int2byte(SMBUS.I2C_SMBUS_READ), ffi.cast('__u8', cmd), arg, data):
            raise IOError(ffi.errno)
        return smbus_data_to_list(data)
 
    @validate(addr=int, cmd=int, vals=list)
    def write_i2c_block_data(self, addr, cmd, vals):
        """write_i2c_block_data(addr, cmd, vals)
         
        Perform I2C Block Write transaction.
        """
        self._set_addr(addr)
        data = ffi.new('union i2c_smbus_data *')
        list_to_smbus_data(data, vals)
        if SMBUS.i2c_smbus_access(self._fd, int2byte(SMBUS.I2C_SMBUS_WRITE), ffi.cast('__u8', cmd), SMBUS.I2C_SMBUS_I2C_BLOCK_BROKEN, data):
            raise IOError(ffi.errno)
 
    @property
    def pec(self):
        return self._pec
 
    @pec.setter
    def pec(self, value):
        """True if Packet Error Codes (PEC) are enabled"""
        pec = bool(value)
        if pec != self._pec:
            if ioctl(self._fd, SMBUS.I2C_PEC, pec):
                raise IOError(ffi.errno)
            self._pec = pec
 
 
def smbus_data_to_list(data):
    block = data.block
    return [ block[i + 1] for i in range(block[0]) ]
 
 
def list_to_smbus_data(data, vals):
    block_max = SMBUS.I2C_SMBUS_BLOCK_MAX
    if len(vals) > block_max or len(vals) == 0:
        raise OverflowError('Third argument must be a list of at least one, but not more than %d integers' % block_max)
    data.block[0] = len(vals)
    for i, val in enumerate(vals):
        data.block[i + 1] = val


from setuptools import setup, find_packages

import os

base_dir = os.path.dirname(__file__)

about = {}
with open(os.path.join(base_dir, "smbus", "__about__.py")) as f:
    exec(f.read(), about)

readme = os.path.join(base_dir, 'README.rst')
with open(readme) as f:
        long_description = f.read()

CFFI_VERSION = '1.1.0'

setup(
    name=about['__title__'],
    version=about['__version__'],
    description=about['__summary__'],
    long_description=long_description,
    author=about['__author__'],
    author_email=about['__email__'],
    url=about['__uri__'],
    license=about['__license__'],
    packages=find_packages(exclude=["test*", "*test*"]),
    zip_safe=False,
    ext_package='smbus',
    install_requires=['cffi >= ' + CFFI_VERSION],
    setup_requires=['cffi >= ' + CFFI_VERSION],
    cffi_modules=["smbus_cffi_build.py:ffi"],

    classifiers=[
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: System Administrators',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: System :: Hardware',
    ],
)



import pytest
from smbus import ffi, list_to_smbus_data, smbus_data_to_list


def test_list_to_smbus_data():
    lst = list(range(10))
    data = ffi.new("union i2c_smbus_data *")
    list_to_smbus_data(data, lst)
    assert data.block[0] == 10
    for i in lst:
        assert data.block[i + 1] == i


def test_smbus_data_to_list():
    lst = list(range(10))
    data = ffi.new("union i2c_smbus_data *")
    list_to_smbus_data(data, lst)
    assert smbus_data_to_list(data) == list(range(10))


def test_list_to_smbus_data_errors():
    data = ffi.new("union i2c_smbus_data *")
    l = list(range(33))
    with pytest.raises(OverflowError):
        list_to_smbus_data(data, l)
    # does not raise
    list_to_smbus_data(data, list(range(32)))


def test_list_to_smbus_full():
    lst = list(range(1, 33))
    data = ffi.new("union i2c_smbus_data *")
    list_to_smbus_data(data, lst)
    assert data.block[0] == len(lst)
    for i in range(len(lst)):
        assert data.block[i + 1] == i + 1
